@@ -1,10 +1,10 @@
 import locale
+from decimal import Decimal
+from typing import Optional, Type
 
 from django.core.exceptions import ObjectDoesNotExist
+from django.core.validators import MinValueValidator
 from django.db import models
-from django.utils.translation import ugettext as _
-
-from openslides.utils.models import MinMaxIntegerField
 
 
 class BaseOption(models.Model):
@@ -16,7 +16,8 @@ class BaseOption(models.Model):
     which has to be a subclass of BaseVote. Otherwise you have to override the
     get_vote_class method.
     """
-    vote_class = None
+
+    vote_class: Optional[Type["BaseVote"]] = None
 
     class Meta:
         abstract = True
@@ -26,7 +27,9 @@ class BaseOption(models.Model):
 
     def get_vote_class(self):
         if self.vote_class is None:
-            raise NotImplementedError('The option class %s has to have an attribute vote_class.' % self)
+            raise NotImplementedError(
+                f"The option class {self} has to have an attribute vote_class."
+            )
         return self.vote_class
 
     def __getitem__(self, name):
@@ -43,7 +46,14 @@ class BaseVote(models.Model):
     Subclasses have to define an option field. This must be a ForeignKeyField
     to a subclass of BasePoll.
     """
-    weight = models.IntegerField(default=1, null=True)  # Use MinMaxIntegerField
+
+    weight = models.DecimalField(
+        default=Decimal("1"),
+        null=True,
+        validators=[MinValueValidator(Decimal("-2"))],
+        max_digits=15,
+        decimal_places=6,
+    )
     value = models.CharField(max_length=255, null=True)
 
     class Meta:
@@ -53,7 +63,7 @@ class BaseVote(models.Model):
         return self.print_weight()
 
     def get_value(self):
-        return _(self.value)
+        return self.value
 
     def print_weight(self, raw=False):
         if raw:
@@ -71,30 +81,46 @@ class CollectDefaultVotesMixin(models.Model):
     Mixin for a poll to collect the default vote values for valid votes,
     invalid votes and votes cast.
     """
-    votesvalid = MinMaxIntegerField(null=True, blank=True, min_value=-2)
-    votesinvalid = MinMaxIntegerField(null=True, blank=True, min_value=-2)
-    votescast = MinMaxIntegerField(null=True, blank=True, min_value=-2)
+
+    votesvalid = models.DecimalField(
+        null=True,
+        blank=True,
+        validators=[MinValueValidator(Decimal("-2"))],
+        max_digits=15,
+        decimal_places=6,
+    )
+    votesinvalid = models.DecimalField(
+        null=True,
+        blank=True,
+        validators=[MinValueValidator(Decimal("-2"))],
+        max_digits=15,
+        decimal_places=6,
+    )
+    votescast = models.DecimalField(
+        null=True,
+        blank=True,
+        validators=[MinValueValidator(Decimal("-2"))],
+        max_digits=15,
+        decimal_places=6,
+    )
 
     class Meta:
         abstract = True
 
-    def append_pollform_fields(self, fields):
-        fields.append('votesvalid')
-        fields.append('votesinvalid')
-        fields.append('votescast')
-        super(CollectDefaultVotesMixin, self).append_pollform_fields(fields)
-
     def get_percent_base_choice(self):
         """
         Returns one of the strings of the percent base.
         """
-        raise NotImplementedError('You have to provide a get_percent_base_choice() method.')
+        raise NotImplementedError(
+            "You have to provide a get_percent_base_choice() method."
+        )
 
 
 class PublishPollMixin(models.Model):
     """
     Mixin for a poll to add a flag whether the poll is published or not.
     """
+
     published = models.BooleanField(default=False)
 
     class Meta:
@@ -109,7 +135,8 @@ class BasePoll(models.Model):
     """
     Base poll class.
     """
-    vote_values = ['Votes']
+
+    vote_values = ["Votes"]
 
     class Meta:
         abstract = True
@@ -122,12 +149,15 @@ class BasePoll(models.Model):
             return True
         return False
 
-    def set_options(self, options_data=[], skip_autoupdate=False):
+    def set_options(self, options_data=None, skip_autoupdate=False):
         """
         Adds new option objects to the poll.
 
         option_data: A list of arguments for the option.
         """
+        if options_data is None:
+            options_data = []
+
         for option_data in options_data:
             option = self.get_option_class()(**option_data)
             option.poll = self
@@ -184,37 +214,11 @@ class BasePoll(models.Model):
             try:
                 vote = self.get_votes().filter(option=option_id).get(value=value)
             except ObjectDoesNotExist:
-                values.append(self.get_vote_class()(value=value, weight=''))
+                values.append(self.get_vote_class()(value=value, weight=""))
             else:
                 values.append(vote)
         return values
 
-    def get_vote_form(self, **kwargs):
-        """
-        Returns the form for one option of the poll.
-        """
-        from openslides.poll.forms import OptionForm
-        return OptionForm(extra=self.get_vote_objects_with_values(kwargs['formid']),
-                          **kwargs)
-
-    def get_vote_forms(self, **kwargs):
-        """
-        Returns a list of forms for the poll.
-        """
-        forms = []
-        for option in self.get_options():
-            form = self.get_vote_form(formid=option.id, **kwargs)
-            form.option = option
-            forms.append(form)
-        return forms
-
-    def append_pollform_fields(self, fields):
-        """
-        Appends additional field to a given list of fields. By default it
-        appends nothing.
-        """
-        pass
-
 
 def print_value(value, percent_base=0):
     """
@@ -222,15 +226,18 @@ def print_value(value, percent_base=0):
     'undocumented' or the vote value with percent value if so.
     """
     if value == -1:
-        verbose_value = _('majority')
+        verbose_value = "majority"
     elif value == -2:
-        verbose_value = _('undocumented')
+        verbose_value = "undocumented"
     elif value is None:
-        verbose_value = _('undocumented')
+        verbose_value = "undocumented"
     else:
         if percent_base:
-            locale.setlocale(locale.LC_ALL, '')
-            verbose_value = u'%d (%s %%)' % (value, locale.format('%.1f', value * percent_base))
+            locale.setlocale(locale.LC_ALL, "")
+            verbose_value = "%d (%s %%)" % (
+                value,
+                locale.format("%.1f", value * percent_base),
+            )
         else:
-            verbose_value = u'%s' % value
+            verbose_value = value
     return verbose_value
