@@ -1,26 +1,25 @@
 import smtplib
-from random import choice
 
+from django.conf import settings
 from django.contrib.auth.hashers import make_password
-from django.contrib.auth.models import Group as DjangoGroup
-from django.contrib.auth.models import GroupManager as _GroupManager
 from django.contrib.auth.models import (
     AbstractBaseUser,
     BaseUserManager,
+    Group as DjangoGroup,
+    GroupManager as _GroupManager,
     Permission,
     PermissionsMixin,
 )
 from django.core import mail
-from django.core.exceptions import ValidationError
+from django.core.exceptions import ObjectDoesNotExist, ValidationError
 from django.db import models
-from django.db.models import Prefetch, Q
+from django.db.models import Prefetch
 from django.utils import timezone
 from jsonfield import JSONField
 
 from ..core.config import config
-from ..core.models import Projector
-from ..utils.collection import CollectionElement
-from ..utils.models import RESTModelMixin
+from ..utils.auth import GROUP_ADMIN_PK
+from ..utils.models import CASCADE_AND_AUTOUPDATE, RESTModelMixin
 from .access_permissions import (
     GroupAccessPermissions,
     PersonalNoteAccessPermissions,
@@ -33,19 +32,24 @@ class UserManager(BaseUserManager):
     Customized manager that creates new users only with a password and a
     username. It also supports our get_full_queryset method.
     """
+
     def get_full_queryset(self):
         """
         Returns the normal queryset with all users. In the background all
         groups are prefetched from the database together with all permissions
         and content types.
         """
-        return self.get_queryset().prefetch_related(Prefetch(
-            'groups',
-            queryset=Group.objects
-                          .select_related('group_ptr')
-                          .prefetch_related(Prefetch(
-                              'permissions',
-                              queryset=Permission.objects.select_related('content_type')))))
+        return self.get_queryset().prefetch_related(
+            Prefetch(
+                "groups",
+                queryset=Group.objects.select_related("group_ptr").prefetch_related(
+                    Prefetch(
+                        "permissions",
+                        queryset=Permission.objects.select_related("content_type"),
+                    )
+                ),
+            )
+        )
 
     def create_user(self, username, password, skip_autoupdate=False, **kwargs):
         """
@@ -56,28 +60,22 @@ class UserManager(BaseUserManager):
         user.save(skip_autoupdate=skip_autoupdate, using=self._db)
         return user
 
-    def create_or_reset_admin_user(self):
+    def create_or_reset_admin_user(self, skip_autoupdate=False):
         """
         Creates an user with the username 'admin'. If such a user already
         exists, resets it. The password is (re)set to 'admin'. The user
-        becomes member of the group 'Staff'. The two important permissions
-        'users.can_see_name' and 'users.can_manage' are added to this group,
-        so that the admin can manage all other permissions.
+        becomes member of the group 'Admin'.
         """
-        query_can_see_name = Q(content_type__app_label='users') & Q(codename='can_see_name')
-        query_can_manage = Q(content_type__app_label='users') & Q(codename='can_manage')
-
-        admin_group, _ = Group.objects.get_or_create(name='Admin')
-        admin_group.permissions.add(Permission.objects.get(query_can_see_name))
-        admin_group.permissions.add(Permission.objects.get(query_can_manage))
-
-        admin, created = self.get_or_create(
-            username='admin',
-            defaults={'last_name': 'Administrator'})
-        admin.default_password = 'admin'
+        created = False
+        try:
+            admin = self.get(username="admin")
+        except ObjectDoesNotExist:
+            admin = self.model(username="admin", last_name="Administrator")
+            created = True
+        admin.default_password = "admin"
         admin.password = make_password(admin.default_password)
-        admin.save()
-        admin.groups.add(admin_group)
+        admin.save(skip_autoupdate=skip_autoupdate)
+        admin.groups.add(GROUP_ADMIN_PK)
         return created
 
     def generate_username(self, first_name, last_name):
@@ -88,12 +86,13 @@ class UserManager(BaseUserManager):
         last_name = last_name.strip()
 
         if first_name and last_name:
-            base_name = ' '.join((first_name, last_name))
+            base_name = " ".join((first_name, last_name))
         else:
             base_name = first_name or last_name
             if not base_name:
-                raise ValueError("Either 'first_name' or 'last_name' must not be "
-                                 "empty.")
+                raise ValueError(
+                    "Either 'first_name' or 'last_name' must not be " "empty."
+                )
 
         if not self.filter(username=base_name).exists():
             generated_username = base_name
@@ -101,21 +100,13 @@ class UserManager(BaseUserManager):
             counter = 0
             while True:
                 counter += 1
-                test_name = '%s %d' % (base_name, counter)
+                test_name = f"{base_name} {counter}"
                 if not self.filter(username=test_name).exists():
                     generated_username = test_name
                     break
 
         return generated_username
 
-    def generate_password(self):
-        """
-        Generates a random passwort. Do not use l, o, I, O, 1 or 0.
-        """
-        chars = 'abcdefghijkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ23456789'
-        size = 8
-        return ''.join([choice(chars) for i in range(size)])
-
 
 class User(RESTModelMixin, PermissionsMixin, AbstractBaseUser):
     """
@@ -124,78 +115,57 @@ class User(RESTModelMixin, PermissionsMixin, AbstractBaseUser):
     in other OpenSlides apps like motion submitter or (assignment) election
     candidates.
     """
+
     access_permissions = UserAccessPermissions()
 
-    USERNAME_FIELD = 'username'
+    USERNAME_FIELD = "username"
 
-    username = models.CharField(
-        max_length=255,
-        unique=True,
-        blank=True)
+    username = models.CharField(max_length=255, unique=True, blank=True)
 
-    first_name = models.CharField(
-        max_length=255,
-        blank=True)
+    first_name = models.CharField(max_length=255, blank=True)
 
-    last_name = models.CharField(
-        max_length=255,
-        blank=True)
+    last_name = models.CharField(max_length=255, blank=True)
+
+    gender = models.CharField(max_length=255, blank=True)
 
     email = models.EmailField(blank=True)
 
-    last_email_send = models.DateTimeField(
-        blank=True,
-        null=True)
+    last_email_send = models.DateTimeField(blank=True, null=True)
 
     # TODO: Try to remove the default argument in the following fields.
 
-    structure_level = models.CharField(
-        max_length=255,
-        blank=True,
-        default='')
+    structure_level = models.CharField(max_length=255, blank=True, default="")
 
-    title = models.CharField(
-        max_length=50,
-        blank=True,
-        default='')
+    title = models.CharField(max_length=50, blank=True, default="")
 
-    number = models.CharField(
-        max_length=50,
-        blank=True,
-        default='')
+    number = models.CharField(max_length=50, blank=True, default="")
 
-    about_me = models.TextField(
-        blank=True,
-        default='')
+    about_me = models.TextField(blank=True, default="")
 
-    comment = models.TextField(
-        blank=True,
-        default='')
+    comment = models.TextField(blank=True, default="")
 
-    default_password = models.CharField(
-        max_length=100,
-        blank=True,
-        default='')
+    default_password = models.CharField(max_length=100, blank=True, default="")
 
-    is_active = models.BooleanField(
-        default=True)
+    is_active = models.BooleanField(default=True)
 
-    is_present = models.BooleanField(
-        default=False)
+    is_present = models.BooleanField(default=False)
 
-    is_committee = models.BooleanField(
-        default=False)
+    is_committee = models.BooleanField(default=False)
 
     objects = UserManager()
 
     class Meta:
         default_permissions = ()
         permissions = (
-            ('can_see_name', 'Can see names of users'),
-            ('can_see_extra_data', 'Can see extra data of users (e.g. present and comment)'),
-            ('can_manage', 'Can manage users'),
+            ("can_see_name", "Can see names of users"),
+            (
+                "can_see_extra_data",
+                "Can see extra data of users (e.g. present and comment)",
+            ),
+            ("can_change_password", "Can change its own password"),
+            ("can_manage", "Can manage users"),
         )
-        ordering = ('last_name', 'first_name', 'username', )
+        ordering = ("last_name", "first_name", "username")
 
     def __str__(self):
         # Strip white spaces from the name parts
@@ -204,7 +174,7 @@ class User(RESTModelMixin, PermissionsMixin, AbstractBaseUser):
 
         # The user has a last_name and a first_name
         if first_name and last_name:
-            name = ' '.join((self.first_name, self.last_name))
+            name = " ".join((self.first_name, self.last_name))
         # The user has only a first_name or a last_name or no name
         else:
             name = first_name or last_name or self.username
@@ -217,29 +187,21 @@ class User(RESTModelMixin, PermissionsMixin, AbstractBaseUser):
         Overridden method to skip autoupdate if only last_login field was
         updated as it is done during login.
         """
-        if kwargs.get('update_fields') == ['last_login']:
-            kwargs['skip_autoupdate'] = True
-            CollectionElement.from_instance(self)
+        if kwargs.get("update_fields") == ["last_login"]:
+            kwargs["skip_autoupdate"] = True
         return super().save(*args, **kwargs)
 
-    def delete(self, skip_autoupdate=False, *args, **kwargs):
-        """
-        Customized method to delete an user. Ensures that a respective
-        user projector element is disabled.
-        """
-        Projector.remove_any(
-            skip_autoupdate=skip_autoupdate,
-            name='users/user',
-            id=self.pk)
-        return super().delete(skip_autoupdate=skip_autoupdate, *args, **kwargs)  # type: ignore
-
     def has_perm(self, perm):
         """
         This method is closed. Do not use it but use openslides.utils.auth.has_perm.
         """
-        raise RuntimeError('Do not use user.has_perm() but use openslides.utils.auth.has_perm')
+        raise RuntimeError(
+            "Do not use user.has_perm() but use openslides.utils.auth.has_perm"
+        )
 
-    def send_invitation_email(self, connection, subject, message, skip_autoupdate=False):
+    def send_invitation_email(
+        self, connection, subject, message, skip_autoupdate=False
+    ):
         """
         Sends an invitation email to the users. Returns True on success, False on failiure.
         May raise an ValidationError, if something went wrong.
@@ -251,32 +213,63 @@ class User(RESTModelMixin, PermissionsMixin, AbstractBaseUser):
         # no error is raised and this is replaced with ''.
         class format_dict(dict):
             def __missing__(self, key):
-                return ''
+                return ""
 
-        message_format = format_dict({
-            'name': str(self),
-            'event_name': config['general_event_name'],
-            'url': config['users_pdf_url'],
-            'username': self.username,
-            'password': self.default_password})
-        message = message.format(**message_format)
+        message_format = format_dict(
+            {
+                "name": str(self),
+                "event_name": config["general_event_name"],
+                "url": config["users_pdf_url"],
+                "username": self.username,
+                "password": self.default_password,
+            }
+        )
+        try:
+            message = message.format(**message_format)
+        except KeyError as err:
+            raise ValidationError({"detail": "Invalid property {0}", "args": [err]})
 
-        subject_format = format_dict({'event_name': config['general_event_name']})
-        subject = subject.format(**subject_format)
+        subject_format = format_dict(
+            {"event_name": config["general_event_name"], "username": self.username}
+        )
+        try:
+            subject = subject.format(**subject_format)
+        except KeyError as err:
+            raise ValidationError({"detail": "Invalid property {0}", "args": [err]})
 
         # Create an email and send it.
-        email = mail.EmailMessage(subject, message, config['users_email_sender'], [self.email])
+        email = mail.EmailMessage(
+            subject,
+            message,
+            config["users_email_sender"] + " <" + settings.DEFAULT_FROM_EMAIL + ">",
+            [self.email],
+            reply_to=[config["users_email_replyto"]],
+        )
         try:
             count = connection.send_messages([email])
         except smtplib.SMTPDataError as e:
             error = e.smtp_code
-            helptext = ''
-            if error == 554:
-                helptext = ' Is the email sender correct?'
+            helptext = ""
+            if error == 554:  # The server does not accept our connection. The code is
+                # something like "transaction failed" or "No SMTP service here"
+                helptext = " Is the email sender correct?"
             connection.close()
-            raise ValidationError({'detail': 'Error {}. Cannot send email.{}'.format(error, helptext)})
+            raise ValidationError(
+                {
+                    "detail": "Error {0}. Cannot send email.{1}",
+                    "args": [error, helptext],
+                }
+            )
         except smtplib.SMTPRecipientsRefused:
             pass  # Run into returning false later
+        except smtplib.SMTPAuthenticationError as e:
+            # Nice error message on auth failure
+            raise ValidationError(
+                {
+                    "detail": "Error {0}: Authentication failure. Please contact your local administrator.",
+                    "args": [e.smtp_code],
+                }
+            )
         else:
             if count == 1:
                 self.email_send = True
@@ -286,27 +279,43 @@ class User(RESTModelMixin, PermissionsMixin, AbstractBaseUser):
 
         return False
 
+    @property
+    def session_auth_hash(self):
+        """
+        Returns the session auth hash of a user as attribute.
+
+        Needed for the django rest framework.
+        """
+        return self.get_session_auth_hash()
+
 
 class GroupManager(_GroupManager):
     """
     Customized manager that supports our get_full_queryset method.
     """
+
     def get_full_queryset(self):
         """
         Returns the normal queryset with all groups. In the background all
         permissions with the content types are prefetched from the database.
         """
-        return (self.get_queryset()
-                    .select_related('group_ptr')
-                    .prefetch_related(Prefetch(
-                        'permissions',
-                        queryset=Permission.objects.select_related('content_type'))))
+        return (
+            self.get_queryset()
+            .select_related("group_ptr")
+            .prefetch_related(
+                Prefetch(
+                    "permissions",
+                    queryset=Permission.objects.select_related("content_type"),
+                )
+            )
+        )
 
 
 class Group(RESTModelMixin, DjangoGroup):
     """
     Extend the django group with support of our REST and caching system.
     """
+
     access_permissions = GroupAccessPermissions()
     objects = GroupManager()
 
@@ -318,12 +327,13 @@ class PersonalNoteManager(models.Manager):
     """
     Customized model manager to support our get_full_queryset method.
     """
+
     def get_full_queryset(self):
         """
         Returns the normal queryset with all personal notes. In the background all
         users are prefetched from the database.
         """
-        return self.get_queryset().select_related('user')
+        return self.get_queryset().select_related("user")
 
 
 class PersonalNote(RESTModelMixin, models.Model):
@@ -331,13 +341,18 @@ class PersonalNote(RESTModelMixin, models.Model):
     Model for personal notes (e. g. likes/stars) of a user concerning different
     openslides objects like motions.
     """
+
     access_permissions = PersonalNoteAccessPermissions()
 
+    personalized_model = True
+    """
+    Each model belongs to one user. This relation is set during creation and
+    will not be changed.
+    """
+
     objects = PersonalNoteManager()
 
-    user = models.OneToOneField(
-        User,
-        on_delete=models.CASCADE)
+    user = models.OneToOneField(User, on_delete=CASCADE_AND_AUTOUPDATE)
     notes = JSONField()
 
     class Meta:
